"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseNIK = void 0;
const address_1 = require("./address");
const luxon_1 = require("luxon");
class ParseNIK {
    constructor(NIK) {
        this.NIK = NIK;
        this.provinsi = address_1.findById('provinsi', this.parseNIK(0, 2)).name;
        this.kota = address_1.findById('kota', this.parseNIK(0, 4)).name;
        this.kecamatan = address_1.findById('kecamatan', this.parseNIK(0, 6));
        this.date = this.getDateFull();
        this.gender = this.getGender();
        this.month = this.getMonthFull();
        this.year = this.getYearFull();
    }
    parseNIK(start, end) {
        return parseInt(this.NIK.substr(start, end));
    }
    getCurrentYear() {
        return parseInt(new Date().getFullYear().toString().substr(-2));
    }
    getYearFull() {
        const nikYear = this.parseNIK(10, 2);
        const currentYear = this.getCurrentYear();
        return nikYear < currentYear
            ? `20${nikYear > 10 ? nikYear : '0' + nikYear.toString()}`
            : `19${nikYear > 10 ? nikYear : '0' + nikYear.toString()}`;
    }
    getMonthFull() {
        let date = this.parseNIK(8, 2);
        return date > 10 ? String(date) : `0${date}`;
    }
    getNIKDate() {
        return this.parseNIK(6, 2);
    }
    getDateFull() {
        let date = this.getNIKDate();
        if (date > 40) {
            date -= 40;
        }
        return date > 10 ? date.toString() : `0${date}`;
    }
    getGender() {
        return this.getNIKDate() > 40 ? "PEREMPUAN" : "LAKI-LAKI";
    }
    getDateString() {
        return luxon_1.DateTime.fromFormat(`${this.date}/${this.month}/${this.year}`, 'dd/MM/yyyy');
    }
    //Get unique code in NIK
    getUniqueCode() {
        return String(this.NIK.substr(12, 4));
    }
    parse() {
        if (!this.validate())
            throw new Error('NIK Invalid');
        return {
            provinsi: this.provinsi,
            kota: this.kota,
            kecamatan: this.kecamatan.name,
            kodepos: String(this.kecamatan.postal_code),
            kelamin: this.gender,
            lahir: this.getDateString().toFormat('dd MMMM yyyy', { locale: 'id-ID' }),
            umur: Math.abs(Math.trunc(this.getDateString().diffNow('years').years)),
            uniqcode: this.getUniqueCode(),
        };
    }
    validate() {
        return this.NIK.length === 16 && !!this.provinsi && !!this.kota && !!this.kecamatan.name;
    }
}
function parseNIK(nik) {
    if (typeof nik == 'number')
        nik = String(nik);
    return new ParseNIK(nik).parse();
}
exports.parseNIK = parseNIK;
